/*********************************************************************
 * FileName:        user.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         sprut
 * Copyright:       2007-2010 Joerg Bredendiek (sprut)
 *
 *
 ********************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#define	NO_CDC	//MCD
//#define	CDC_HEX	// nonsens



/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include "system\typedefs.h"
#include "system\usb\usb.h"
#include "io_cfg.h"             // I/O pin mapping

#include "delays.h"
//#include "system\interrupt\interrupt.h"
#include "user\user.h"
#include "timers.h"


/** V A R I A B L E S ********************************************************/
#pragma udata

byte				counter;
DATA_PACKET 		dataPacket;


// IO-Pins
byte				MaskeA		= 0x7F;  // welche Portbits duerfen manipuliert werden?
byte				MaskeB		= 0xFF;
byte				MaskeC		= 0xC7;

//ADC
byte				AcdMaskeA	= 0;	// von ADC verwendete Pins, eigentlich nicht mehr verwendet
byte				RelaisisUsed=0;		//Wenn ein relais eingeschaltet wurde. Benutzt fr Methode ClearRelais


/** D E C L A R A T I O N S **************************************************/
#pragma code
//Aufruf nach Reset
void UserInit(void)
{
	// init pins
	MaskeA	= 0x7F;  // welche Portbits duerfen manipuliert werden?
	MaskeB	= 0xFF;
	MaskeC	= 0xC7;

//Alle Ports als EINGNGE setzen:
	TRISA 	= 0xFF;
	TRISB 	= 0xFF;
	TRISC 	= 0xFF;


}//end UserInit




/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{   
    // User Application USB tasks
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;
    ServiceRequests();
}//end ProcessIO






// Kommando auswerten und ausfhren **********************
void ServiceRequests(void)
{
    byte	index;
	word	big_counter;
	WORD	uidaten;
  
    // wurde via USB etwas empfangen?
	#ifdef NO_CDC	// MCD Treiber
    if(USBGenRead((byte*)&dataPacket,sizeof(dataPacket)))  //byte USBGenRead(byte *buffer, byte len)	//normal
	#endif	//NO_CDC

    {
        counter = 0;
		switch(dataPacket.CMD) 		//das ist in Assembler ein riesieger Sprungverteiler 
        {
			// eine Kennummer aus dem PIC auslesen
            case READ_VERSION:
                //dataPacket._byte[1] is len
                dataPacket._byte[2] = MINOR_VERSION; // Firmware-Version steht in user.h
                dataPacket._byte[3] = MAJOR_VERSION; // Hardware-Version
                counter=0x04;
                break;

			//**********************************************************************//
			// IO, ADC

			case SYS_IO:
				UP_IO();
   				break;

			case SYS_ADC:
				UP_ADC();
   				break;

             
            case RESET:
                //When resetting, make sure to drop the device off the bus
                //for a period of time. Helps when the device is suspended.
                UCONbits.USBEN = 0;
                big_counter = 0;		//word = 16 Bit
                while(--big_counter);
				SleepMs(250);
				SleepMs(250);

                Reset();
                break;
                
            default:
				dataPacket._byte[0] = 0xFF;		// Error, unbekannter Befehl
				counter = 16;
                break;
        }//end switch()


		#ifdef NO_CDC		//senden mit microchip custom driver
		if(counter != 0)
		{
			if(!mUSBGenTxIsBusy())
				USBGenWrite((byte*)&dataPacket,counter);
		}
		#endif


    }	//end if wurde via USB entwas empfangen 




}//end ServiceRequests


/*************

0 - System/Interface
1 - 0=aus/1=init/2=write/3=read/...
2 - daten
3 - daten
4 - daten
5 - daten
6 - daten
7 - daten

**************/


//*********************************************************************************************//

//** I O - P I N S ****************************************************************************//

// IO-Ports A..C
// 0 - aus:   alle Pin auf input
// 1 - init:  TRIS ganzer ports und pull-up
// 2 - write: schreiben ganzer ports
// 3 - read:  lesen ganzer ports

// 4 - setzen einzelner TRIS-Bits (die mit 1)
// 5 - loeschen einzelner TRIS-Bits (die mit 1)
// 6 - setzen einzelner Pins (die mit 1)
// 7 - loeschen einzelner pins (die mit 1)
void UP_IO(void)
{
	switch (dataPacket._byte[1])
	{
		case 0://aus:   alle Pin auf input
			TRISA |= MaskeA;
			TRISB |= MaskeB;
			TRISC |= MaskeC;
			break;
		case 1://init:  TRIS ganzer ports und pull-up
			TRISA = dataPacket._byte[2] & MaskeA;
			TRISB = dataPacket._byte[3] & MaskeB;
			TRISC = dataPacket._byte[4] & MaskeC;
			if (dataPacket._byte[5] & 1 == 1)	INTCON2bits.NOT_RBPU = 0; //pullup an
				else 							INTCON2bits.NOT_RBPU = 1; //pullup aus
			break;
		case 2://write: schreiben ganzer ports
			LATA = (LATA & (~MaskeA)) | (dataPacket._byte[2] & MaskeA);
			LATB = (LATB & (~MaskeB)) | (dataPacket._byte[3] & MaskeB);
			LATC = (LATC & (~MaskeC)) | (dataPacket._byte[4] & MaskeC);
			break;
		case 3://read:  lesen ganzer ports
			dataPacket._byte[2] = PORTA;
			dataPacket._byte[3] = PORTB;
			dataPacket._byte[4] = PORTC;
			break;
		case 4://setzen einzelner TRIS-Bits (die mit 1)
			TRISA |= (dataPacket._byte[2] & MaskeA);
			TRISB |= (dataPacket._byte[3] & MaskeB);
			TRISC |= (dataPacket._byte[4] & MaskeB);
			break;
		case 5://loeschen einzelner TRIS-Bits (die mit 1)
			TRISA &= ~(dataPacket._byte[2] & MaskeA);
			TRISB &= ~(dataPacket._byte[3] & MaskeB);
			TRISC &= ~(dataPacket._byte[4] & MaskeB);
			break;
		case 6://setzen einzelner Pins (die mit 1)
			LATA |= (dataPacket._byte[2] & MaskeA);
			LATB |= (dataPacket._byte[3] & MaskeB);
			LATC |= (dataPacket._byte[4] & MaskeC);
			break;
		case 7://loeschen einzelner pins (die mit 1)
			LATA &= ~(dataPacket._byte[2] & MaskeA);
			LATB &= ~(dataPacket._byte[3] & MaskeB);
			LATC &= ~(dataPacket._byte[4] & MaskeC);
			break;
	}
	counter = 16;
} // UP_IO



//** A D C *******************************************************************************//

// ADC mit AN0..4, 10 Bit, Ref=Vdd/Vss
//Case-Flle:
// 0 - ADC aus
// 1 - Zahl der Eingaenge einschalten, Vref festlegen
// 2 - aktuellen Eingang  0..10 festlegen
// 3 - Resultat auslesen
void UP_ADC(void)
{
	switch (dataPacket._byte[1])
	{
		case 0://ADC aus
			ADCON0    = 0x00; 		// AN1, ADC-aus
			ADCON1    = 0x0F; 		// alle Port-Pins sind jetzt digital
			//MaskeA   |= AcdMaskeA;	// Freigabe der verwendeten ADC-Pins
			//AcdMaskeA = 0x00;
			break;
		
		case 1: //ADC an, Eingaenge festlegen:
			ADCON0 = 0x00; 		    // ADC-aus
			ADCON2 = 0xAA;		    // right-justified, Aquisi=12Tad, Fad=Fosc/32 (schnell)

//Es werden folgende AD-Kanle bentigt:
//0,1,2,3,4 = Port A, dazu TRISA als Eingnge schalten
//8,9,10= Port B. Achtung: In Port B werden auch I/O- Kanle benutzt! 
//TRISB entsprechend als Ausgnge/ Eingnge setzen!

			ADCON1 = 0x04;//RA0-RA5 und RB1-RB4 sind jetzt gltige Analogeingnge

//Bentigte Pins von Port A als Eingang setzen:
			TRISA	|= 0x2F;//=0010 1111 Pin RA0, RA1, RA2, RA3 und RA5 sind AD-Eingnge A0-A5. 
							//Achtung: RA4 hat keinen Mglichkeit als AD-Eingang zu arbeiten!

			
//Bentigte Pins von Port B als Eingang setzen:
			TRISB	|= 0xE;//=1110 RB1,RB2,RB3 sind jetzt die weiteren Eingnge fr den AD-Wandler A8,A9,A10


//Referenzquelle (wird fixiert auf Betriebspannung, ich verzichte auf eine Auswahl, daher auskommentiert)
/*			switch (dataPacket._byte[3])
			{
				case 1:	ADCON1  |= 0x10; break;		//AN3 = ref+
				case 2:	ADCON1  |= 0x20; break;		//AN2 = ref-
				case 3:	ADCON1  |= 0x30; break;		//AN2 = ref-	AN3 = ref+
				default:                 break;
			}
*/

			ADCON0=0x01; 		             // AN0, ADC- insgesamt erlaubt (enabled), Konvertierung wird aber noch nicht gestartet!
			break;



		case 2: //zu Konvertierenden Eingang festlegen und Wandlung starten. "dataPacket._byte[3]" hat die Nummer des AD-Eingangs
			switch (dataPacket._byte[2])
			{
				//A0: Spannung Dnnschichtzelle
				case 0:	ADCON0 = 0; ADCON0 |= 0x01; break; //A0 ist Eingang, Start Konvertierung
				
				//A1: Strom Dnnschichtzelle
				case 1: ADCON0 |= 0x04; SetRelais(dataPacket._byte[2]); ADCON0 |= 0x1; break;//A1 ist Eingang fr AD-Wandler, Relais wird eingeschaltet, danach Start Konvertierung

				//A2: Spannung Polyzelle
				case 2:	ADCON0 |= 0x08; ADCON0 |= 0x01; break; //A2 ist Eingang fr AD-Wandler, Start Konvertierung
				
				//A3: Strom Polyzelle
				case 3: ADCON0 |= 0x0C; SetRelais(dataPacket._byte[2]); ADCON0 |= 0x01; break;//A3 ist Eingang fr AD-Wandler, Relais wird eingeschaltet, danach Start Konvertierung

				//A4: Spannung Monozelle
				case 4:	ADCON0 |= 0x10; ADCON0 |= 0x01; break; //A4 ist Eingang fr AD-Wandler, Start Konvertierung
				
				//A10: Strom Monozelle
				case 10: ADCON0 |= 0x28; SetRelais(dataPacket._byte[2]); ADCON0 |= 0x01; break;//A10 ist Eingang fr AD-Wandler, Relais wird eingeschaltet, danach Start Konvertierung


				//A8: Temperatursensor
				case 8: ADCON0 |= 0x20; ADCON0 |= 0x01; break;//A8 ist Eingang fr AD-Wandler, Start Konvertierung

				//A9: Lichtsensor
				case 9: ADCON0 |= 0x24; ADCON0 |= 0x01; break;//A9 ist Eingang fr AD-Wandler, Start Konvertierung
			}

			//ADCON0 = (dataPacket._byte[2]<<2) | 1;
			//warten
			SleepUs(40);//AD-Wandler Zeit geben
			break;

		case 3: //Resultat auslesen
			ADCON0bits.GO = 1;              // Start AD conversion
	   		while(ADCON0bits.NOT_DONE);     // Wait for conversion
            dataPacket._byte[2] = ADRESL;
            dataPacket._byte[3] = ADRESH;
			
			ClearRelais();//Relais Ausschalten, Messung beendet
			break;
	}
	counter = 16;
} // UP_ADC



//Routinen zum Setzen/ Lschen der Relais
//Alle Relais Ausschalten, egal ob sie vorher Eingeschaltet waren.
//Es betrifft die Port-Pins RB4, RB5, RB6 
void ClearRelais(void)
{
	if (RelaisisUsed == 0) return; //Es war kein Relais eingeschaltet

//Betroffene Pins auf Ausgang schalten, Vorsichtsmanahme	
	TRISB &= 0xF1; //F1= 11110001. Damit RB4,RB5,RB6= 0
	LATB &= 0xF1;
	
	RelaisisUsed = 0;

	//ein wenig warten fr den Abfall der Relais:
	SleepUs(200);
}

//Schaltet ein gewnschtes Relais ein. Die Nummer des relais wird bergeben.
//Es gilt folgende Zuordnung:
//1:  Korrospondiert mit AD-Eingang 1, Portpin RB4
//3:  Korrospondiert mit AD-Eingang 3, Portpin RB5
//10: Korrospondiert mit AD-Eingang 10, Portpin RB6
void SetRelais(byte k)
{
	switch(k)
	{
		case 1:TRISB &= 0x10; 	LATB |= 0x10; RelaisisUsed = 1; break; //RB4 EIN
		case 3:TRISB &= 0x20; 	LATB |= 0x20; RelaisisUsed = 1; break; //RB5 EIN
		case 10:TRISB &= 0x40; 	LATB |= 0x40; RelaisisUsed = 1; break; //RB6 EIN
	}
	SleepMs(3);
}


/****  W A R T E S C H L E I F E N **************************************************************/

// verzgerungen bis max. 63 ms
// Resonator 20 MHz, CPU-Takt 48 MHz, Cycl. 12 MHz
// 1 us = 12 Zyklen
// 1 ms = 12*1000 Zyklen
// High-Teil x 3072
// Low Teil x 12
// die delay-Routinen vertragen als input nur  1..255
void SleepUs(unsigned int us)
{
	byte teilzeit;
	teilzeit = us & 0x00FF;
    if (teilzeit>0)
	{
		Delay10TCYx(teilzeit);		// 10 Zyklen ~ 1s
	}
	teilzeit = us >> 8;			// 1/256
    if (teilzeit>0)
	{
		Delay1KTCYx(teilzeit);	// 3070 Zyklen ~258s ~0,25ms
		Delay1KTCYx(teilzeit);
		Delay1KTCYx(teilzeit);
		Delay10TCYx(teilzeit);
		Delay10TCYx(teilzeit);
		Delay10TCYx(teilzeit);
		Delay10TCYx(teilzeit);
		Delay10TCYx(teilzeit);
		Delay10TCYx(teilzeit);
		Delay10TCYx(teilzeit);
	}
} //SleepUs



// verzgerungen bis max. 255 ms
// Resonator 20 MHz, CPU-Takt 48 MHz, Cycl. 12 MHz
// 1 us = 12 Zyklen
// 1 ms = 12*1000 Zyklen
// 4 ms = 48x1000 Zyklen
// die delay-Routinen vertragen als input nur  1..255
void SleepMs(unsigned int ms)
{
    if (ms>0)
	{
		Delay10KTCYx(ms);
		Delay1KTCYx(ms);
		Delay1KTCYx(ms);
	}
} //SleepMs



/****  E E P R O M   F U N K T I O N E N  F U E R   S T E U E R P I C ********************/


//lesen von Daten aus dem EEPROM des Steuerpic
//0 - Befehl
//1 - Startadresse -low
//2 - Startadresse high = 0
//3 - Blocklnge
void readEdata(void)
{
	byte	Nummer;
	word	Adresse;
	dataPacket._byte[0] = READ_EDATA;
	counter =63;
	Nummer = 4;
	for (Adresse=dataPacket._byte[1]; Adresse<dataPacket._byte[1]+dataPacket._byte[3]; Adresse++)
	{
		EECON1bits.EEPGD = 0;				/* READ step #1 */
		EEADR = Adresse;					/* READ step #2 */
		EECON1bits.RD = 1;					/* READ step #3 */
		dataPacket._byte[Nummer] = EEDATA;	/* READ step #4 */
		Nummer += 1;
	}
	/**** EEPROM read
	MOVLW	DATA_EE_ADDR 	;
	MOVWF	EEADR 			; Lower bits of Data Memory Address to read
	BCF		EECON1, EEPGD 	; Point to DATA memory
	BCF		EECON1, CFGS 	; Access EEPROM
	BSF		EECON1, RD 		; EEPROM Read
	MOVF	EEDATA, W 		; W = EEDATA
	*********/
}


//lesen von Daten aus dem EEPROM des Steuerpic
//0 - Befehl
//1 - Startadresse -low
//2 - Startadresse high = 0
//3 - Blocklnge
//4 - 1. Datenbyte
void writeEdata(void)
{
	byte	Nummer;
	word	Adresse;
	counter = 0x01;
	dataPacket._byte[0] = WRITE_EDATA;
	Nummer = 4;
	for (Adresse=dataPacket._byte[1]; Adresse<dataPacket._byte[1]+dataPacket._byte[3]; Adresse++)
	{
		EECON1bits.EEPGD = 0;				/* WRITE step #1 */
		EECON1bits.WREN = 1;				/* WRITE step #2 */
		EEADR = Adresse;					/* WRITE step #3 */
		EEDATA = dataPacket._byte[Nummer];	/* WRITE step #4 */
		EECON2 = 0x55;						/* WRITE step #5 */
		EECON2 = 0xaa;						/* WRITE step #6 */
		EECON1bits.WR = 1;					/* WRITE step #7 */
		while (!PIR2bits.EEIF)				/* WRITE step #8 */
			;
		PIR2bits.EEIF = 0;					/* WRITE step #9 */
		Nummer+=1;
	}
}

	/**** EEPROM write
	MOVLW 	DATA_EE_ADDR 	;
	MOVWF 	EEADR 			; Lower bits of Data Memory Address to write
	MOVLW 	DATA_EE_DATA 	;
	MOVWF 	EEDATA 			; Data Memory Value to write
	BCF 	EECON1, EPGD 	; Point to DATA memory
	BCF 	EECON1, CFGS 	; Access EEPROM
	BSF 	EECON1, WREN 	; Enable writes
	BCF 	INTCON, GIE 	; Disable Interrupts
	MOVLW 	55h ;

	;Required Sequence
	MOVWF 	EECON2 			; Write 55h
	MOVLW 	0AAh 			;
	MOVWF 	EECON2 			; Write 0AAh
	BSF 	EECON1, WR 		; Set WR bit to begin write

	BSF 	INTCON, GIE 	; Enable Interrupts
							; User code execution
	BCF 	EECON1, WREN 	; Disable writes on write complete (EEIF set)
	**************/



/****  A D C   F U N K T I O N E N  ********************************************************/

// Spannung an AN1 auslesen *************************
// sollte mit CCP1 synchronisiert werden
void ReadADC(void)
{
	ADCON0bits.GO = 1;              // Start AD conversion
   	while(ADCON0bits.NOT_DONE);     // Wait for conversion
    return;
}//end ReadADC





/** EOF user.c ***************************************************************/
