﻿using System;
using System.Collections.Generic;
using System.Configuration; 
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using DWORD = System.UInt32;

namespace Solar3
{
    public partial class FormMain : Form
    {
        public FormMain()
        {
            InitializeComponent();
        }

        //Meine Instanzen:
        DateTime Startzeit;
        TimeSpan Laufzeit = TimeSpan.Zero;

        PICUSB.usb_interface usbi = new PICUSB.usb_interface();//Zugriff auf USB zum PIC-IC

        decimal SumDS = 0;//Summe elektrische Arbeit in mWs Dünnschichtmodul
        decimal SumPC = 0;//Summe elektrische Arbeit in mWs Polykristallinmodul
        decimal SumMC = 0;//Summe elektrische Arbeit in mWs Monokristallinmodul

        /// <summary>
        /// True wenn der Timer für das Messintervall noch läuft
        /// </summary>
        bool _btimer_Wait_Mess = false;
        bool _bisRunning = false;//True wenn unendliche Messung läuft


        private void FormMain_Load(object sender, EventArgs e)
        {

            //Anfangsinitialisierung:
            this.StripLabel_Info.Text = "Initialisiere";
            Application.DoEvents();

            this.TP_Main.BackColor = System.Drawing.SystemColors.Control;
            this.TP_Tests.BackColor = System.Drawing.SystemColors.Control;
            this.TP_Config.BackColor = System.Drawing.SystemColors.Control;

            //Userwerte lesen und zuordnen:
            this.TB_Uref.Value = Properties.Settings.Default.Referenzspannung;
            
            this.TB_MaxPowerDS.Value = Properties.Settings.Default.MaxPowerDS;
            this.TB_MaxPowerPC.Value = Properties.Settings.Default.MaxPowerPC;
            this.TB_MaxPowerMC.Value = Properties.Settings.Default.MaxPowerMC;

            //Startzeit:
            this.TB_Startzeit.Value = Properties.Settings.Default.Startzeit;

            //Endzeit Messung:
            this.TB_Endzeit.Value = Properties.Settings.Default.Endzeit;

            //Messintervall:
            this.TB_Messinterval.Value = Properties.Settings.Default.Messintervall;

            //Shuntwiderstände für die Strommessung
            this.TB_ShuntDS.Value = Properties.Settings.Default.ShuntDS;
            this.TB_ShuntPC.Value = Properties.Settings.Default.ShuntPC;
            this.TB_ShuntMC.Value = Properties.Settings.Default.ShuntMC;

            //Formfaktoren:
            this.TB_FF_DS.Value = Properties.Settings.Default.FF_DS;
            this.TB_FF_PC.Value = Properties.Settings.Default.FF_PC;
            this.TB_FF_MC.Value = Properties.Settings.Default.FF_MC;

            //Temperaturwerte
            this.TB_U_TempRef.Value = Properties.Settings.Default.Temp_Ref;
            this.TB_Kelvin.Value = Properties.Settings.Default.Temp_Kelvin;
            this.TB_TempRef.Value = Properties.Settings.Default.Temp_RefGrad;


            //Timer für aktuelles Datum und Uhrzeit starten:
            this.timer_aktuelleZeit.Enabled = true;
            this.timer_aktuelleZeit.Start();

            this.StripLabel_Info.Text = "";
        }

        /// <summary>
        /// abgelaufene Zeit anzeigen:
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer_Laufzeit_Tick(object sender, EventArgs e)
        {
            string s;
            Laufzeit=this.GetMEZ().Subtract(this.Startzeit);
            s = Laufzeit.ToString();
            s = s.Substring(0, s.IndexOf('.'));
            this.StripLabel_RunTime.Text = s;
        }



        private void BT_Ende_Click(object sender, EventArgs e)
        {
            //this.Close();
            this.timer_Laufzeit.Stop();
            this.timer_Laufzeit.Enabled = false;
            this.timer_Wait_Mess.Stop();
            this.timer_Wait_Mess.Enabled = false;
            this.timer_aktuelleZeit.Stop();
            this.timer_aktuelleZeit.Enabled = false;
            this._bisRunning = false;
            this._btimer_Wait_Mess = false;
            Application.DoEvents();

            Application.Exit(); 
        }

        /// <summary>
        /// Start der Messung:
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_StartMessung_Click(object sender, EventArgs e)
        {

            this.BT_StopMessung.Enabled = true;
            this.BT_StartMessung.Enabled = false;
            this.StripLabel_Info.Text = "Starte Messung";
            this.TB_Temp.Text = "? °C";
            Application.DoEvents();


            this.Startzeit = this.GetMEZ();
            StripLabel_Startzeit.Text = this.Startzeit.ToShortTimeString();

            this._bisRunning = true;//Messung läuft

            //Timer für das Messintervall setzen:
            this.timer_Wait_Mess.Interval = ((int)TB_Messinterval.Value) * 1000; //Sekunden nach Millisekunden

            //Timer für die Laufzeit starten:
            this.timer_Laufzeit.Enabled = true;
            this.timer_Laufzeit.Start();

            this._btimer_Wait_Mess = true;
            this.timer_Wait_Mess.Enabled = true;
            this.timer_Wait_Mess.Start();

            //Messung aller Module beginnen:

            while (this._bisRunning)
            {
                DateTime dat = this.GetMEZ();
                if ((dat.Hour >= (int)this.TB_Startzeit.Value) & (dat.Hour <= (int)TB_Endzeit.Value))
                {

                    if (!MessenundAnzeigen())
                    {
                        this._bisRunning = false;
                        this._btimer_Wait_Mess = true;
                        this.timer_Wait_Mess.Stop();
                    }
                    else
                    {
                        //Warten bis Messinterval abgelaufen ist:
                        StripLabel_Info.Text = "Warte auf nächste Messung";
                        while (this._btimer_Wait_Mess) { Application.DoEvents(); }

                        ////Messinterval-Timer wieder starten:
                        this._btimer_Wait_Mess = true;
                    }
                }
                else
                {
                    this.StripLabel_Info.Text = "Es ist Nacht, keine Messung";
                }

                Application.DoEvents();
            }

        }



        DS_SolarTableAdapters.SolarTableAdapter ta = new Solar3.DS_SolarTableAdapters.SolarTableAdapter();
        DS_Solar.SolarDataTable dt = new DS_Solar.SolarDataTable();
        DS_Solar.SolarRow dr = null;

        /// <summary>
        /// Liest alle Wert der Module und zeigt sie an
        /// </summary>
        bool MessenundAnzeigen()
        {
            decimal u, i, p;
            

            //Wenn Messwerte gespeichert werden sollen:
            //if (CB_MessungAufzeichnen.Checked)
            //{
                dr = dt.NewSolarRow();
            //}

            //---------------> Zuerst Dünnschichtmodul:
            //Spannung und Kurzschlusstrom holen:
            if (!this.GetValuesDS(out u, out i)) { return false; };
            p = u * i;

            dr.DS_Strom = i;
            dr.DS_Spannung = u;
            


            //Formfaktor berücksichtigen:
            p = p * this.TB_FF_DS.Value / 100.0m;

            dr.DS_Power = p;


            this.L_U_DS_aktuell.Text = u.ToString("0.0V");
            this.L_I_DS_aktuell.Text = i.ToString("0.0mA");
            this.L_Pm_DS_aktuell.Text = p.ToString("0.0mW");

            //Elekrische Arbeit berechnen (Einheit mWs), dazu Messzeit berücksichtigen:
            this.SumDS = this.SumDS + p * TB_Messinterval.Value;

            string sEinheit = "mWs";
            decimal anzeig=SumDS;

            //Bisherige Summe elektrischer Arbeit formatiert anzeigen. Wenn Wert zu groß, dann nach Ws bzw. Wh umrechnen:
            if (anzeig > 1000.0m) { anzeig = anzeig / 1000.0m; sEinheit = "Ws"; }//Von mWs auf Ws umformen;
            if (anzeig > 4000.0m) { anzeig = anzeig / 3600.0m; sEinheit = "Wh"; }

            //Anzeigen:
            this.TB_DS_PowerSum.Text = string.Concat(anzeig.ToString("0.0 "), sEinheit);

            //Schieberegler stellen:
            //Momentane Leistung in Prozent von der maximalen Leistung berechnen:
            int val = 0;
            p = u * i;
            p = p * this.TB_FF_DS.Value / 100.0m;
            if (TB_MaxPowerDS.Value < p)
            {
                //Momentane Leistung größer als maximal vorgegeben.
                TB_MaxPowerDS.Value = (int)p;
                Properties.Settings.Default.MaxPowerDS = (int)p;
                Properties.Settings.Default.Save();
            }
            
            this.TBar_DS.Value = (int)((p / this.TB_MaxPowerDS.Value) * 100.0m);
            dr.DS_PowerProz = this.TBar_DS.Value;


            this.L_U_DS_Proz.Text = string.Concat(this.TBar_DS.Value.ToString("0"), " %");




            //---------------> Jetzt Polykristallinmodul:
            if (!this.GetValuesPC(out u, out i)) { return false; };
            p = u * i;

            //Formfaktor berücksichtigen:
            p = p * this.TB_FF_PC.Value / 100.0m;

            dr.PC_Spannung = u;
            dr.PC_Strom = i;
            dr.PC_Power = p;


            this.L_U_PC_aktuell.Text = u.ToString("0.0V");
            this.L_I_PC_aktuell.Text = i.ToString("0.0mA");
            this.L_Pm_PC_aktuell.Text = p.ToString("0.0mW");

            //Elekrische Arbeit berechnen (Einheit mWs), dazu Messzeit berücksichtigen:
            this.SumPC = this.SumPC + p * TB_Messinterval.Value;

            sEinheit = "mWs";
            anzeig = SumPC;

            //Bisherige Summe elektrischer Arbeit formatiert anzeigen. Wenn Wert zu groß, dann nach Ws bzw. Wh umrechnen:
            if (anzeig > 1000.0m) { anzeig = anzeig / 1000.0m; sEinheit = "Ws"; }//Von mWs auf Ws umformen;
            if (anzeig > 4000.0m) { anzeig = anzeig / 3600.0m; sEinheit = "Wh"; }

            //Anzeigen:
            this.TB_PC_PowerSum.Text = string.Concat(anzeig.ToString("0.0 "), sEinheit);

            //Schieberegler stellen:
            //Momentane Leistung in Prozent von der maximalen Leistung berechnen:
            val = 0;
            p = u * i;
            p = p * this.TB_FF_PC.Value / 100.0m;
            if (TB_MaxPowerPC.Value < p)
            {
                //Momentane Leistung größer als maximal vorgegeben.
                TB_MaxPowerPC.Value = (int)p;
                Properties.Settings.Default.MaxPowerPC = (int)p;
                Properties.Settings.Default.Save();
            }

            val = (int)((p / this.TB_MaxPowerPC.Value) * 100.0m);
            this.TBar_PC.Value = val;
            dr.PC_Power_Proz = val;
            
            this.L_U_PC_Proz.Text = string.Concat(this.TBar_PC.Value.ToString("0"), " %");
            
            
            //---------------> Jetzt Monokristallinmodul:

            if (!this.GetValuesMC(out u, out i)) { return false; };
            p = u * i;

            //Formfaktor berücksichtigen:
            p = p * this.TB_FF_MC.Value / 100.0m;

            dr.MC_Spannung = u;
            dr.MC_Strom = i;
            dr.MC_Power = p;

            this.L_U_MC_aktuell.Text = u.ToString("0.0V");
            this.L_I_MC_aktuell.Text = i.ToString("0.0mA");
            this.L_Pm_MC_aktuell.Text = p.ToString("0.0mW");

            //Elekrische Arbeit berechnen (Einheit mWs), dazu Messzeit berücksichtigen:
            this.SumMC = this.SumMC + p * TB_Messinterval.Value;

            sEinheit = "mWs";
            anzeig = SumMC;

            //Bisherige Summe elektrischer Arbeit formatiert anzeigen. Wenn Wert zu groß, dann nach Ws bzw. Wh umrechnen:
            if (anzeig > 1000.0m) { anzeig = anzeig / 1000.0m; sEinheit = "Ws"; }//Von mWs auf Ws umformen;
            if (anzeig > 4000.0m) { anzeig = anzeig / 3600.0m; sEinheit = "Wh"; }

            //Anzeigen:
            this.TB_MC_PowerSum.Text = string.Concat(anzeig.ToString("0.0 "), sEinheit);

            //Schieberegler stellen:
            //Momentane Leistung in Prozent von der maximalen Leistung berechnen:
            val = 0;
            p = u * i;
            p = p * this.TB_FF_MC.Value / 100.0m;



            if (TB_MaxPowerMC.Value < p)
            {
                //Momentane Leistung größer als maximal vorgegeben.
                TB_MaxPowerMC.Value = (int)p;
                Properties.Settings.Default.MaxPowerPC = (int)p;
                Properties.Settings.Default.Save();

            }

            val = (int)((p / this.TB_MaxPowerMC.Value) * 100.0m);
            this.TBar_MC.Value = val;

            dr.MC_Power_Proz = val;

            this.L_U_MC_Proz.Text = string.Concat(this.TBar_MC.Value.ToString("0"), " %");



            //----Lichtstärke:
            //decimal dLicht;
            //if (!this.GetSpannung(6, out dLicht)) return false;

            //Anzeigen:
            //this.TB_Licht.Text = ((int)dLicht * 1000.0m).ToString();
 
            //TBar_Licht.Value = (int)(((dLicht / TB_MaxValueLicht.Value)*1000.0m)*100.0m);  

            //Temperatur:
            decimal sp;
            int iTemp=0;
            if (!GetSpannung(8, out sp))
            {
                return false;
            }
            iTemp=this.CalcTemperature(sp);
            this.TB_Temp.Text = iTemp.ToString("0 °C");
            
            dr.Licht = 0.0m;
            dr.Temp = iTemp;


            dr.Datum = this.GetMEZ();
            TimeSpan ts;
            TimeSpan.TryParse(string.Format("{0}:{1}:{2}",
                this.GetMEZ().Hour.ToString(), this.GetMEZ().Minute.ToString(), this.GetMEZ().Second.ToString()), out ts);

            dr.Messzeit = ts;

            dt.Rows.Add(dr);


            //in Datenbank speichern:
            if (CB_MessungAufzeichnen.Checked)
            {
                ta.Update(dt);
            }
            dt.Clear();
            dt.AcceptChanges();
            
            Application.DoEvents();
            return true;
        }

        # region "Testsseite"

        /// <summary>
        /// Zeige Versionsnummer der Firmware
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_ReadVersionsnummer_Click(object sender, EventArgs e)
        {
            byte[] send_buf = new byte[1];
            byte[] get_buf = new byte[4];

            send_buf[0] = 0;
            usbi.EasyCommand(1, 4, send_buf, out get_buf);//1= ein Byte senden, 4= 4 Bytes empfangen

            if (get_buf == null)
            {
                //USB-Fehler!
                MessageBox.Show("USB-Fehler! Der Puffer get_buf ist null", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            L_FW_Nummer.Text = string.Format("Firmwareversion: {0}.{1}", get_buf[3], get_buf[2]);
        }

        /// <summary>
        /// Ermittelt die Anzahl angeschlossener USB-Geräte mit der PID von SPRUT:
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_devCount_Click(object sender, EventArgs e)
        {
            uint x;
            x = this.usbi.GetDeviceCount(usbi.vid_pid_norm);

            L_AnzahlDev.Text = x.ToString();  //this.usbi.GetDeviceCount(usbi.vid_pid_norm).ToString();  
        }

        private void BT_DLL_Vers_Click(object sender, EventArgs e)
        {
            uint x = usbi.DLLVersion;
            l_DLLVersion.Text = x.ToString("X");
        }

        /// <summary>
        /// Zeige Spannung Dünnschichtmodul
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_U_DSModul_Click(object sender, EventArgs e)
        {
            decimal sp;
            if (!this.GetSpannung(0, out sp))
            {
                this.L_U_DSModul.Text = "Fehler";
                return;
            }
            this.L_U_DSModul.Text = sp.ToString("0.0V");
        }

        /// <summary>
        /// Zeige Strom Dünnschichtmodul
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_I_DSModul_Click(object sender, EventArgs e)
        {
            decimal sp;
            GetSpannung(1, out sp);
            sp = sp * 1000m;//Ampere nach mA konvertieren
            this.L_I_DSModul.Text = sp.ToString("0.0mA");
        }


        /// <summary>
        /// Zeige Spannung Polykristalinmodul
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_U_PCModul_Click(object sender, EventArgs e)
        {
            decimal sp;
            if (!this.GetSpannung(2, out sp))
            {
                this.L_U_PCModul.Text = "Fehler";
                return;
            }
            this.L_U_PCModul.Text = sp.ToString("0.0V");
        }

        /// <summary>
        /// Zeige Strom Polymodul
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_I_PCModul_Click(object sender, EventArgs e)
        {
            decimal sp;
            if (!GetSpannung(4, out sp))
            {
                this.L_I_PCModul.Text = "Fehler";
                return;
            }
            sp = sp * 1000m;//Ampere nach mA konvertieren
            this.L_I_PCModul.Text = sp.ToString("0.0mA");
        }

        /// <summary>
        /// Zeige Spannung Monokristalinmodul
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_U_MCModul_Click(object sender, EventArgs e)
        {
            decimal sp;
            if (!GetSpannung(10, out sp))
            {
                this.L_U_MCModul.Text = "Fehler";
                return;
            }

            this.L_U_MCModul.Text = sp.ToString("0.0V");
        }


        /// <summary>
        /// Zeige Strom Mono-Modul
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_I_MCModul_Click(object sender, EventArgs e)
        {
            decimal sp;
            if (!GetSpannung(10, out sp, 1))
            {
                this.L_I_MCModul.Text = "Fehler";
                return;
            }
            sp = sp * 1000m;//Ampere nach mA konvertieren
            this.L_I_MCModul.Text = sp.ToString("0.0mA");
        }


        /// <summary>
        /// Spannung Lichtsensor anzeigen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_Temp_Grad_Click(object sender, EventArgs e)
        {
            decimal sp;
            if (!GetSpannung(8, out sp))
            {
                this.L_TempGrad.Text = "Fehler";
                return;
            }
            this.L_TempGrad.Text = this.CalcTemperature(sp).ToString("0 °C");
        }

        /// <summary>
        /// Zeigt Spannung Temperatursensor
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_Temp_Click(object sender, EventArgs e)
        {
            decimal sp;
            if (!GetSpannung(8, out sp))
            {
                this.L_Temp.Text = "Fehler";
                return;
            }

            this.L_Temp.Text = sp.ToString("0.00V");
        }

# endregion

        /// <summary>
        /// Überlagerung. Von allen Messungen außer Strom Monozelle benutzt
        /// </summary>
        /// <param name="Kanal"></param>
        /// <returns></returns>
        private bool GetSpannung(byte Kanal, out decimal Spannung)
        {
            return GetSpannung(Kanal, out Spannung, 0);
        }

        /// <summary>
        /// Ermittelt anliegende Spannung am übergebenen Kanal, Ausgabewert in mV
        /// Wenn Strommessung Monozelle: StrommessungMono=1
        /// Wenn USB-Fehler auftritt, Return = False
        /// </summary>
        /// <param name="Kanal"></param>
        /// <param name="Spannung"></param>
        /// <param name="StrommessungMono"></param>
        /// <returns></returns>
        private bool GetSpannung(byte Kanal, out decimal Spannung, byte StrommessungMono)
        {
            byte[] send_buf = new byte[20];
            byte[] get_buf = new byte[20];
            int i;

            for (i = 0; i < 20; i++) { send_buf[i] = 0; }
            for (i = 0; i < 20; i++) { get_buf[i] = 0; }

            uint res;
 
            
            //AD-Wandler initialisieren, alle AD-Wandler- Portpins als Eingang setzen:
            send_buf[0] = 0x51;   // ADC
            send_buf[1] = 1;      // init
            send_buf[2] = 0;      // nicht benutzt
            send_buf[3] = 0;      // ref

            res=usbi.EasyCommand(16, 16, send_buf, out get_buf);//Befehl ausführen
            if (res != 1) 
            {
                MessageBox.Show("Fehler in USB-Übertragung! \nAD-Wandler konnte nicht initialisiert werden!", 
                    "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Spannung = 0.0m;
                return false;
            }

            //Gewünschten AD-Eingang setzen, Wandlung starten:
            send_buf[0] = 0x51;
            send_buf[1] = 2;
            send_buf[2] = Kanal;
            //Sonderfall Strom Monozelle:
            send_buf[3] = StrommessungMono;//Nur 1 wenn Strommessung Monozelle

            res = usbi.EasyCommand(16,16 , send_buf, out get_buf);//Befehl ausführen
            if (res != 1) 
            { 
                MessageBox.Show("Fehler in USB-Übertragung! \nAD-Wandler-Kanal konnte nicht gesetzt werden!", 
                "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Spannung=0.0m;
                return false;
            }

            //Ermittelten Wert abholen:
            send_buf[0] = 0x51;
            send_buf[1] = 3;
            send_buf[2] = 0;
            send_buf[3] = 0;
            res=usbi.EasyCommand(16,16, send_buf, out get_buf);//Befehl ausführen

            if (res != 1) 
            { MessageBox.Show("Fehler in USB-Übertragung! \nWandlungsergebnis konnte nicht gelesen werden!", 
                "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            Spannung = 0.0m;
            return false;
            }

            //auf [2]= LOWbyte, auf [3]= HIGHByte
           
            int  Wert=get_buf[3];
            Wert= Wert * 256;
            Wert= Wert+get_buf[2];
            //Den Wert nun nach echte Volt konvertieren, dazu Referenzspannung benutzen:

            
            Spannung = Wert * (TB_Uref.Value/4096m);//12-Bitt Auflösung

            //Übersetzungsverhältnis des zuständigen OV's mit einrechnen und damit reelen Wert berechnen
            decimal uv = 0.0m;

            switch (Kanal)
            {
                case 0: uv = 3m;//Spannung Dünnschichtzelle
                    break;
                case 1: uv = 1.0m;//Strom DS-Zelle
                    break;
                case 2: uv = 3m;//Spannung Polyzelle
                    break;
                case 4: uv = 1.0m;//Strom Polyzelle
                    break;
                case 10: uv = 1.0m;//Strom und Spannung Monozelle
                    break;
                case 8: uv = 1.68m;//Spannung Temperatursensor
                    break;

                
                default: uv = 1.0m;
                    break;
            }

            //Falls Strommessung, so noch den Lastwiderstand berücksichtigen!
            switch (Kanal)
            {
                case 1: uv = uv / this.TB_ShuntDS.Value;
                    break;
                case 4: uv = uv / this.TB_ShuntPC.Value;
                    break;
                case 10: if (StrommessungMono==1) { uv = uv / this.TB_ShuntMC.Value; }
                    break;
            }

            //Rückgabe Einheit Spannung: Volt!
            // Eineit für Strom: mA!
            Spannung= Spannung * uv;
            return true;
        }

        /// <summary>
        /// Berechnet die Temperatur in ganzahligen Grad auf Grundlage der Referenzwerte aus der Konfiguration
        /// Übergebene Spannung in Volt!
        /// </summary>
        /// <param name="U_Temp"></param>
        /// <returns></returns>
        private int CalcTemperature(decimal U_Temp)
        {
            int temp = 0;
            decimal d;
            d = U_Temp - this.TB_U_TempRef.Value;
            d = d * 1000.0m;//Von Volt in mV umrechnen
            d = d / this.TB_Kelvin.Value;
            temp = (int)(this.TB_TempRef.Value + d);
            return temp;
        }


        /// <summary>
        /// Userwerte speichern
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_SaveConfig_Click(object sender, EventArgs e)
        {
            this.BT_SaveConfig.Enabled = false;
            Application.DoEvents();

            //Referenzspannung AD-Wandler:
            Properties.Settings.Default.Referenzspannung = this.TB_Uref.Value;
            
            //Maximale Leistung Dünnschichtmodul für Anzeige 100%
            Properties.Settings.Default.MaxPowerDS = (int)this.TB_MaxPowerDS.Value;
            //Maximale Leistung Polykristallinmodul für Anzeige 100%
            Properties.Settings.Default.MaxPowerPC = (int)this.TB_MaxPowerPC.Value;
            //Maximale Leistung Monokristallinmodul für Anzeige 100%
            Properties.Settings.Default.MaxPowerMC = (int)this.TB_MaxPowerMC.Value;

            //Startzeit Dauermessung:
            Properties.Settings.Default.Startzeit = (int)this.TB_Startzeit.Value;
            Properties.Settings.Default.Endzeit = (int)this.TB_Endzeit.Value;
            Properties.Settings.Default.Messintervall  = (int)this.TB_Messinterval.Value;

            //Shuntwiderstände für die Strommessung
            Properties.Settings.Default.ShuntDS = this.TB_ShuntDS.Value;
            Properties.Settings.Default.ShuntPC = this.TB_ShuntPC.Value;
            Properties.Settings.Default.ShuntMC = this.TB_ShuntMC.Value;

            //Formfaktoren:
            Properties.Settings.Default.FF_DS = (int)this.TB_FF_DS.Value;
            Properties.Settings.Default.FF_PC = (int)this.TB_FF_PC.Value;
            Properties.Settings.Default.FF_MC = (int)this.TB_FF_MC.Value;

            //Temperaturwerte
            Properties.Settings.Default.Temp_Ref = this.TB_U_TempRef.Value;
            Properties.Settings.Default.Temp_Kelvin = this.TB_Kelvin.Value;
            Properties.Settings.Default.Temp_RefGrad = this.TB_TempRef.Value;

            
            //TB_TempRef.Value;


            Properties.Settings.Default.Save();

            this.BT_SaveConfig.Enabled = true;
        }



        /// <summary>
        /// Liefert momentane Leistung Dünnschichtmodul
        /// </summary>
        /// <returns></returns>
        bool GetValuesDS(out decimal u, out decimal i)
        {
            bool b= true;
            i = 0.0m;
            b=this.GetSpannung(0, out u);//Spannung in V
            if (!b) { return b; }
            b = this.GetSpannung(1, out i);//Strom in A
            i = i * 1000.0m;//von Ampere nach mA konvertieren
            return b;
        }

        /// <summary>
        /// Liefert momentane Leistung Polymodul
        /// </summary>
        /// <returns></returns>
        bool GetValuesPC(out decimal u, out decimal i)
        {
            bool b = true;
            i = 0.0m;
            b = this.GetSpannung(2, out u);//Spannung in V
            if (!b) { return b; }
            b = this.GetSpannung(4, out i);//Strom in A
            i = i * 1000.0m;//von Ampere nach mA konvertieren
            return b;
        }

        /// <summary>
        /// Liefert momentane Leistung Monomodul
        /// </summary>
        /// <returns></returns>
        bool GetValuesMC(out decimal u, out decimal i)
        {
            bool b = true;
            i = 0.0m;
            b = this.GetSpannung(10, out u);//Spannung in V
            if (!b) { return b; }
            b = this.GetSpannung(10, out i, 1);//Strom in A
            i = i * 1000.0m;//von Ampere nach mA konvertieren
            return b;
        }

        /// <summary>
        /// Timer für Messintervall abgelaufen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer_Wait_Mess_Tick(object sender, EventArgs e)
        {
            this._btimer_Wait_Mess = false;
        }

        /// <summary>
        /// Stopp Messung
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BT_StopMessung_Click(object sender, EventArgs e)
        {
            this._bisRunning = false;
            this.BT_StartMessung.Enabled = true;
            this.timer_Laufzeit.Stop();
            this.StripLabel_Info.Text = "Messung gestoppt";
        }




        /// <summary>
        /// Aktuelles Datum und Zeit anzeigen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer_aktuelleZeit_Tick(object sender, EventArgs e)
        {
            StripLabel_Uhrzeit.Text = this.GetMEZ().ToShortTimeString();
            //Aktuelles Datum:
            this.StripLabel_Datum.Text = this.GetMEZ().ToShortDateString();
//            timer_aktuelleZeit.Start();
        }


        /// <summary>
        /// Liefert Datum+Uhrzeit in MEZ
        /// </summary>
        /// <returns></returns>
        DateTime GetMEZ()
        {
            return (DateTime.UtcNow.AddHours(1));
        }




        
    }
}
